import os

import cv2

from cadivi_analysis.common.config_util import ConfigUtil
from cadivi_analysis.common import config_const as ConfigConst
from cadivi_analysis.utils.image_utils import write_frame
from cadivi_analysis.error_detector.error_detector import ErrorDetector


def main():
    config_util = ConfigUtil()
    error_detector = ErrorDetector()

    input_image_path = config_util.get_property(
        section=ConfigConst.YOLOV8, key=ConfigConst.IMAGE_DEMO
    )
    output_image_path = config_util.get_property(
        section=ConfigConst.YOLOV8, key=ConfigConst.OUTPUT_IMAGE_PATH
    )
    bentoml_url = config_util.get_property(
        section=ConfigConst.YOLOV8, key=ConfigConst.BENTOML_URL
    )
    # conf_minimum = config_util.get_float(
    #     section=ConfigConst.YOLOV8,
    #     key=ConfigConst.CONF_MINIMUM,
    #     default_val=ConfigConst.DEFAULT_CONF,
    # )

    # Read Image
    images = []
    image = cv2.imread(input_image_path)
    images.append(image)

    # Save Image
    image_paths = []
    os.makedirs(output_image_path, exist_ok=True)
    write_frame(image, f"{output_image_path}/output_image.png")
    image_paths.append(f"{output_image_path}/output_image.png")

    # Call bentoml yolo
    try:
        data = error_detector.request_yolo_bentoml(image_paths, bentoml_url)
        print(data)
    except Exception as e:
        print(e)

    # Draw Output
    output_images = error_detector.draw_bboxes(images, data)
    for img in output_images:
        write_frame(img, f"{output_image_path}/output_image.png")



import json
from typing import List, Any, Dict

import cv2
import requests


class ErrorDetector:
    def request_yolo_bentoml(
        self, images: List[str], bentoml_url: str
    ) -> List[Dict[Any, Any]]:
        files = [
            (
                "images",
                ("file", open(f"{image}", "rb"), "application/octet-stream"),
            )
            for image in images
        ]

        headers = {"accept": "application/json"}

        response = requests.request(
            "POST", bentoml_url, headers=headers, files=files
        )

        if response.status_code != 200:
            raise Exception(f"Failed {response.status_code}: {response.text}")

        data = json.loads(response.text)
        return data

    def draw_bboxes(self, images, error_detections):
        output_images = []
        for image, error_list in zip(images, error_detections):
            for error_dict in error_list:
                # Draw Bounding Boxes
                x1, y1, x2, y2 = (
                    error_dict["box"]["x1"],
                    error_dict["box"]["y1"],
                    error_dict["box"]["x2"],
                    error_dict["box"]["y2"],
                )
                cv2.putText(
                    image,
                    f"{error_dict['name']} {error_dict['confidence']:.2f}",
                    (int(x1), int(y1) - 10),
                    cv2.FONT_HERSHEY_SIMPLEX,
                    0.9,
                    (0, 0, 255),
                    2,
                )
                cv2.rectangle(
                    image,
                    (int(x1), int(y1)),
                    (int(x2), int(y2)),
                    (0, 0, 255),
                    2,
                )
            output_images.append(image)
        return output_images
